package com.tcs.tool.filter;

import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

public class HTTPAccessFilter implements Filter {
	private FileWriter accessLogFile = null;
	private boolean browserTimeCaptureEnabled = false;
	private FileWriter clientSideExecutionTimeFile = null;

	public void init(FilterConfig filterConfig) throws ServletException {
		System.out.println("HTTPAccessFilter is loaded...");
		try {
			this.accessLogFile = new FileWriter(filterConfig.getInitParameter("server.time.log.path"));
		} catch (IOException e) {
			throw new ServletException(e);
		}
		String sBrowserTimeCaptureEnabled = filterConfig.getInitParameter("browser.time.log.enabled");
		if ( sBrowserTimeCaptureEnabled != null && "true".equalsIgnoreCase(sBrowserTimeCaptureEnabled) ) {
			this.browserTimeCaptureEnabled = true;
			try {

				this.clientSideExecutionTimeFile = 
					new FileWriter(filterConfig.getInitParameter("browser.time.log.path"));
			} catch (IOException e) {
				e.printStackTrace();
				this.browserTimeCaptureEnabled = false;
				//throw new ServletException(e);
			}
		}
	}

	public void destroy() {}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		long startTime = System.currentTimeMillis();//t1
		chain.doFilter(request, response);
		long endTime = System.currentTimeMillis();//t2

		long deltaTime = (endTime-startTime);//in milliseconds /1000; //in seconds
		HttpServletRequest hReq = (HttpServletRequest) request;
		String jSessionId = hReq.getSession(true).getId();
		String userId = hReq.getRemoteHost();	//for simplicity let us make the userId equals to remoteHostAddress	
		
		//Now dealing with (t2-t1)
		if ( this.accessLogFile != null ) {
			writeServerSideExecutionTime(hReq, userId, jSessionId, deltaTime);
		}

		//Now dealing with (t4-t0)
		if ( this.browserTimeCaptureEnabled && this.clientSideExecutionTimeFile != null )  {
			writeBrowserSideExecutionTime(hReq, userId, jSessionId);
		}

	}

	private void writeServerSideExecutionTime(HttpServletRequest hReq, String userId, 
												String jSessionId, long deltaTime) {

		String remoteAddress = hReq.getRemoteAddr();
		SimpleDateFormat dateFormat = new SimpleDateFormat("[dd/MMM/yyyy:HH:mm:ss]");
		String endDate = dateFormat.format(new Date());
		StringBuffer sb = new StringBuffer();
		sb.append(remoteAddress);
		sb.append(" - - ");
		sb.append(endDate);
		sb.append(' ');
		sb.append(deltaTime);
		sb.append(" \"");
		sb.append(hReq.getMethod());
		sb.append(" [");
		sb.append(hReq.getRequestURI());
		sb.append("] ").append(hReq.getProtocol()).append("\" ");
		sb.append('"');
		sb.append(hReq.getHeader("user-agent"));
		sb.append('"');

		sb.append(" USERID="+userId); 
		if (jSessionId != null) sb.append(" JSESSIONID="+jSessionId);  
		sb.append("\r\n");

		try {
			//writing (t2-t1)
			accessLogFile.write(sb.toString());
			accessLogFile.flush();
    	}catch (Exception e) {
    		//todo: handle properly 
    		e.printStackTrace();
    	}
	}
	
	private void writeBrowserSideExecutionTime(HttpServletRequest hReq, String userId, 
							String jSessionId) {

		String prevPagePostTime = null;
		String pageLoadName = null;
		String pageloadTime = null;

		Cookie cookie1[]= hReq.getCookies();
		if (cookie1 != null) {                                            
			for (int i=0; i<cookie1.length; i++) {                         
				Cookie cookie = cookie1[i];
				if (cookie != null && cookie.getName().equals("pageLoadName"))    
					pageLoadName = cookie.getValue();        

				if (cookie != null && cookie.getName().equals("prevPagePostTime"))    
					prevPagePostTime = cookie.getValue(); 

				if (cookie != null && cookie.getName().equals("pageloadTime"))    
					pageloadTime = cookie.getValue();
			} 
		}
		if ( pageLoadName == null || pageLoadName.equalsIgnoreCase("NULL")) pageLoadName = "UNKNOWN";
		//if ( hReq.getRequestURI().indexOf("/Login.") != -1 ) return;
		if ( prevPagePostTime != null && prevPagePostTime.trim().length() > 0 
			&& !prevPagePostTime.trim().equals("null")) {

			//System.out.println("pageloadTime=" + pageloadTime);
			//System.out.println("pageloadTime=" + prevPagePostTime);
			Date dtPageloadTime = getClientSideDate(pageloadTime);
			if ( dtPageloadTime == null ) return;
			Date dtPrevPagePostTime = getClientSideDate(prevPagePostTime);
			if ( dtPrevPagePostTime == null ) return;
			//t4 - t0
			long executionTime = (dtPageloadTime.getTime() - dtPrevPagePostTime.getTime() );///1000; 
			
			SimpleDateFormat dateFormat = new SimpleDateFormat("[dd/MMM/yyyy:HH:mm:ss]");
			StringBuffer sb = new StringBuffer();
			sb.append(hReq.getRemoteAddr());
			sb.append(" - - ");
			sb.append(dateFormat.format(dtPageloadTime));
			sb.append(" ");
			sb.append(executionTime);
			sb.append(" \"");
			sb.append(hReq.getMethod());
			sb.append(" [");
			sb.append(pageLoadName);
			sb.append("] ").append(hReq.getProtocol()).append("\" ");
			sb.append('"');
			sb.append(hReq.getHeader("user-agent"));
			sb.append('"');
			if (userId != null) sb.append(" USERID="+userId);              
			if (jSessionId != null) sb.append(" JSESSIONID="+jSessionId);   
			sb.append("\r\n");                                            
	       
			try {
				//writing (t4-t0)
        		clientSideExecutionTimeFile.write(sb.toString());                               
        		clientSideExecutionTimeFile.flush();   
        	}catch (Exception e) {
        		// todo: handle properly 
        		e.printStackTrace();
        	}			
						
		}		
	}

	private Date getClientSideDate(String inputTime) {
		if ( inputTime == null ) return null;
		inputTime = inputTime.trim();
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy/M/d HH:mm:ss:SSS");
		try {
			return dateFormat.parse(inputTime);
		} catch (ParseException e) {
			// todo: handle properly 
			e.printStackTrace();
		}		
		return null;
	}

}
